Role('JooseX.Class.Singleton', {
    
    has : {
        forceInstance           : Joose.I.Object,
        instance                : null
    },
    
    
    
    override : {
        
        defaultConstructor : function () {
            var meta        = this
            var previous    = this.SUPER()
            
            this.adaptConstructor(previous)
            
            return function (forceInstance, params) {
                if (forceInstance == meta.forceInstance) return previous.apply(this, params) || this
                
                var instance = meta.instance
                
                if (instance) {
                    if (meta.hasMethod('configure')) instance.configure.apply(instance, arguments)
                } else
                    meta.instance = new meta.c(meta.forceInstance, arguments)
                    
                return meta.instance
            }
        }        
    }
    

})


Joose.Namespace.Manager.my.register('Singleton', Class({
    isa     : Joose.Meta.Class,
    meta    : Joose.Meta.Class,
    
    does    : JooseX.Class.Singleton
}))



/**

Name
====


JooseX.Class.Singleton - a trait, turning your class into singleton


SYNOPSIS
========

        Class("Some.Class", {
        
            trait : JooseX.Class.Singleton,
            
            has : {
                attribute   : null
            },
            
            
            methods : {
            
                configure : function (params) {
                }
            }
        })
        
        var instance1 = new Some.Class({
            attribute : 'value'
        })
        
        var instance2 = new Some.Class()
        
        //preferred way
        var instance3 = Some.Class()
        
        t.ok(instance1 == instance2 && instance2 == instance3, '3 ways to access a singleton instance - choose one')
        
        
        //or equivalent declaration

        Singleton("Some.Class", {
        
            has : {
                attribute   : null
            },
            
            
            methods : {
            
                configure : function (params) {
                    this.attribute = params
                }
            }
        })


DESCRIPTION
===========

`JooseX.Class.Singleton` is a meta-role, turning your class into singleton.

You can apply it as a trait (the 1st example in the synopsys), or you can use the new declaration helper: `Singleton`


USAGE
=====

To receive the singleton instance use the constructor of the class. You can use it in any form - with or without `new` keyword.

        Singleton('Some.Class', {
            ...
        })
        
        var instance = Some.Class()
        
        //or

        var instance = new Some.Class()
        
        
The first call to constructor will *instantiate* singleton. The parameters passed to this first call, will be passed into actual underlaying constructor.

The further calls to constructor will *reconfigure* singleton. The parameters passed to further calls, will be passed into call to `configure` method
of the class (if it has one).


INHERITANCE
===========

Note, that by default, the subclasses uses the same metaclass as their parents. Thus, if you'll subclass the singleton, the resulting class will be 
a singleton either (even without applied traits or `Singleton` helper usage):

        Class('Some.Singleton', {
            trait : JooseX.Class.Singleton,
            ...
        })

        
        // this class will be a singleton as well
        
        Class('Another.Class', {
            isa : Some.Singleton
        })

If you'd like to avoid this - then specify the metaclass explicitly:

        // this class will be a usual class 
        // inheriting all attributes/methods of Some.Singleton
        // but not its "singleton-nessment"

        Class('Another.Class', {
            meta : Joose.Meta.Class,
            
            isa : Some.Singleton
        })


GETTING HELP
============

This extension is supported via github issues tracker: <http://github.com/SamuraiJack/JooseX-Class-Singleton/issues>

For general Joose questions you can also visit #joose on irc.freenode.org or the forum at: <http://joose.it/forum>
 


SEE ALSO
========

Web page of this module: <http://github.com/SamuraiJack/JooseX-Class-Singleton/>

General documentation for Joose: <http://Joose.github.com/Joose>


BUGS
====

All complex software has bugs lurking in it, and this module is no exception.

Please report any bugs through the web interface at <http://github.com/SamuraiJack/JooseX-Class-Singleton/issues>



AUTHORS
=======

Nickolay Platonov <nplatonov@cpan.org>



COPYRIGHT AND LICENSE
=====================

Copyright (c) 2009, Nickolay Platonov

All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
* Neither the name of Nickolay Platonov nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 


*/
;
Role('SymbieX.SomePlugin', {
    
    trait       : 'JooseX.CPS',
    
    
    continued   : {
        
        override : {
            
            PRE    : function (context) {
                ACTION.push('SymbieX.SomePlugin: PRE')
                
                this.SUPER(context).now()
            }
        },
        
        
        after : {
            
            BEGIN : function () {
                ACTION.push('SymbieX.SomePlugin: BEGIN')
                
                this.CONTINUE()
            }
        },
        
        before : {
            
            END     : function () {
                ACTION.push('SymbieX.SomePlugin: END')
                
                this.CONTINUE()
            }
        }
    }
})
;
Class('SymbieX.Controller.FooBar', {
    
    isa : 'Symbie.Controller',
    
    prefix  : 'symbiex',
    
    routes : {
        
        'foobar/*' : {
            
            action      : function () {
                ACTION.push('SymbieX.Controller.FooBar: foobar/*')
                
                this.CONTINUE()
            }
        }
    },
    
    methods : {
        
        ACTIVATE : function () {
            ACTION.push('SymbieX.Controller.FooBar: ACTIVATE')
        },
        

        FINALIZE : function () {
            ACTION.push('SymbieX.Controller.FooBar: FINALIZE')
        }
    },
    
    
    continued : {
        
        methods : {
            
            PRE : function (context) {
                ACTION.push('SymbieX.Controller.FooBar: PRE')
                
                this.SUPER(context).now()
            },
            
    
            BEGIN : function (context) {
                ACTION.push('SymbieX.Controller.FooBar: BEGIN')
                
                this.SUPER(context).now()
            },
            
    
            END : function (context) {
                ACTION.push('SymbieX.Controller.FooBar: END')
                
                this.SUPER(context).now()
            }
        }
    }
    
})

;
Class('TestApp.Controller.WikiController', {
    
    isa : 'Symbie.Controller',
    
    prefix  : 'wiki',
    
    routes : {
        
        INDEX : {
            action      : function () {
                ACTION.push('TestApp.Controller.WikiController: INDEX')
                
                this.CONTINUE()
            }
        },
        
        
        editWiki : {
            map         : 'edit',
            
            action      : function () {
                ACTION.push('TestApp.Controller.WikiController: editWiki')
                
                this.CONTINUE()
            }
        },
        
        
        wikiPage : {
            map         : ':page',
            
            where : {
                page    : /\d+/
            },
            
            action      : function () {
                ACTION.push('TestApp.Controller.WikiController: wikiPage')
                
                this.CONTINUE()
            }
        },
        
        
        catchAll : {
            map         : '*',
            
            action      : function () {
                ACTION.push('TestApp.Controller.WikiController: catchAll')
                
                this.CONTINUE()
            }
        }
    },
    
    
    methods : {
        
        ACTIVATE : function () {
            ACTION.push('TestApp.Controller.WikiController: ACTIVATE')
        },
        

        FINALIZE : function () {
            ACTION.push('TestApp.Controller.WikiController: FINALIZE')
        }
    },
    
    
    continued : {
        
        methods : {
            
            PRE : function (context) {
                ACTION.push('TestApp.Controller.WikiController: PRE')
                
                this.SUPER(context).now()
            },
            
    
            BEGIN : function (context) {
                ACTION.push('TestApp.Controller.WikiController: BEGIN')
                
                this.SUPER(context).now()
            },
            
    
            END : function (context) {
                ACTION.push('TestApp.Controller.WikiController: END')
                
                this.SUPER(context).now()
            }
        }
    }
    
})
;
Class('TestApp.Controller.Pictures', {
    
    isa : 'Symbie.Controller',
    
    controllers     : [
        'TestApp.Controller.WikiController',
        
        {
            'SymbieX.Controller.FooBar' : {
                prefix  : 'foobar/prefix'
            }
        }
    ],
    
    routes : {
        
        './all/:fromDate/:toDate' : {
            
            where : {
                fromDate    : /(\d\d)-(\d\d)-(\d{4})/,
                toDate      : /(\d\d-\d\d-\d{4})/
            },
            
            action      : function (c, fromDate, toDate) {
                ACTION.push('TestApp.Controller.Pictures: ./all/:fromDate/:toDate')
                
                var t = c.dispatchParams.t
                
                if (t) {
                    
                    t.ok(fromDate == c.getParam('fromDate'),    'Correct param #1')
                    t.ok(fromDate[1] == '12',                   'Correct param #1')
                    t.ok(fromDate[2] == '34',                   'Correct param #1')
                    t.ok(fromDate[3] == '1234',                 'Correct param #1')
                    
                    t.ok(toDate == c.getParam('toDate'),        'Correct param #2')
                    t.ok(toDate == '56-78-5678',                'Correct param #2')
                    
                    t.ok(c.app() == TestApp(), 'Correct value for `c.app()`')
                    
                    t.ok(c.getRoute() == TestApp.Controller.Pictures.meta.getRoute('./all/:fromDate/:toDate'), 'Correct route in context')
                    t.isaOk(c.controller, TestApp.Controller.Pictures, 'Correct controller in context')
                    
                    t.ok(this == c.controller, "Action is running in the controller's scope")
                }
                
                this.CONTINUE()
            }
        },
        
        
        '/glob/picture' : {
            
            where : {
                id    : /\d+/
            },
            
            action      : function () {
                ACTION.push('TestApp.Controller.Pictures: /glob/picture')
                
                this.CONTINUE()
            }
        },
        
        
        editPicture : {
            map : './:id/edit',
            
            where : {
                id    : /\d+/
            },
            
            action      : function () {
                ACTION.push('TestApp.Controller.Pictures: ./:id/edit')
                
                this.CONTINUE()
            }
        },
        
        
        wiki : function () {
            this.CONTINUE()
        }
    },
    
    
    methods : {
        
        ACTIVATE : function () {
            ACTION.push('TestApp.Controller.Pictures: ACTIVATE')
        },
        

        FINALIZE : function () {
            ACTION.push('TestApp.Controller.Pictures: FINALIZE')
        }
    },
    
    
    continued : {
        
        methods : {
            
            PRE : function (context) {
                ACTION.push('TestApp.Controller.Pictures: PRE')
                
                this.SUPER(context).now()
            },
            
    
            BEGIN : function (context) {
                ACTION.push('TestApp.Controller.Pictures: BEGIN')
                
                this.SUPER(context).now()
            },
            
    
            END : function (context) {
                ACTION.push('TestApp.Controller.Pictures: END')
                
                this.SUPER(context).now()
            }
        }
    }
    
})

;
Class('TestApp', {
    
    isa             : 'Symbie.Application',
    
    trait           : 'JooseX.Class.Singleton',
    
    controllers     : [
        'TestApp.Controller.WikiController',
        
        {
            'TestApp.Controller.Pictures' : {
                prefix  : '/pictures'
            },
            
            'SymbieX.Controller.FooBar' : {
                prefix  : 'barbaz'
            }
        }
    ],
    
    
    plugins         : {
        'SymbieX.SomePlugin' : {
            option1     : 'value1',
            option2     : 'value2'
        }
    },
    
    
    routes : {
        
        '/' : {
            action      : function () {
                ACTION.push('TestApp: /')
                
                this.CONTINUE()
            }
        },
        
        
        home : {
            map       : '/home',
            
            action      : function () {
                ACTION.push('TestApp: /home')
                
                this.CONTINUE()
            }
        }
    },
    
    
    methods : {
        
        ACTIVATE : function () {
            ACTION.push('TestApp: ACTIVATE')
        },
        

        FINALIZE : function () {
            ACTION.push('TestApp: FINALIZE')
        }
    },
    
    
    continued : {
        
        methods : {
            
            PRE : function (context) {
                ACTION.push('TestApp: PRE')
                
                if (window.TEST_PRE) {
                    this.CONTINUE(false)
                    
                    return
                }
                
                this.SUPER(context).now()
            },
            
    
            BEGIN : function (context) {
                ACTION.push('TestApp: BEGIN')
                
                this.SUPER(context).now()
            },
            
    
            END : function (context) {
                ACTION.push('TestApp: END')
                
                this.SUPER(context).now()
            }
        }
    }
    
})
;
Class('Symbie.Meta.Route.Token', {
    
    has : {
        token               : null,
        
        specificity         : 0
    },
    
    
    methods : {
        
        match : function (token) {
            throw "Abstract method 'match' was called on [" + this + "]"
        },
        
        
        asString : function () {
            throw "Abstract method 'asString' was called on [" + this + "]"
        }
    }
    
})


;
Class('Symbie.Meta.Route.Token.Parameter', {
    
    isa : 'Symbie.Meta.Route.Token',
    
    has : {
        name                : null,
        
        specificity         : 50,
        
        regex               : { required : true }
    },
    
    
    methods : {
        
        match : function (token) {
            return this.regex.exec(token)
        },
        
        
        asString : function (parameters) {
            var parameterValue = parameters && parameters[this.name]
            
            if (!parameterValue) throw "Parameter [" + this.token + "] wasn't supplied during token stringification"
            
            if (!this.regex.test(parameterValue)) throw "Supplied parameter [" + parameterValue + "] don't match the required regular expression in 'where'"
            
            return parameterValue
        }
    }
    
})


;
Class('Symbie.Meta.Route.Token.String', {
    
    isa : 'Symbie.Meta.Route.Token',
    
    has : {
        specificity         : 100
    },
    
    
    methods : {
        
        match : function (token) {
            return token == this.token
        },
        
        
        asString : function () {
            return this.token
        }
    }
    
})


;
Class('Symbie.Meta.Route.Token.Root', {
    
    isa : 'Symbie.Meta.Route.Token.String',
    
    has : {
        token               : ''
    }
    
})


;
Class('Symbie.Meta.Route.Token.WildCard', {
    
    isa : 'Symbie.Meta.Route.Token',
    
    has : {
        token               : '*',
        
        specificity         : 25
    },
    
    
    methods : {
        
        match : function (token) {
            return true
        },
        
        
        asString : function (parameters) {
            var splat       = parameters.splat
            
            if (splat instanceof Array) return splat.join('/')
            
            return ''
        }
    }
    
})


;
Class('Symbie.Meta.Route', {
    
    meta    : Joose.Meta.Class,
    
    isa     : Joose.Managed.Property,
        
    trait   : 'JooseX.CPS',
    
    use     : [ 
        'Symbie.Meta.Route.Token.String', 
        'Symbie.Meta.Route.Token.Root', 
        'Symbie.Meta.Route.Token.Parameter', 
        'Symbie.Meta.Route.Token.WildCard' 
    ],
    
    
    has : {
        action                  : { required : true },
        
        map                     : null,
        
        where                   : null,
        
        
        
        tokens                  : null,
        use                     : null,
        ready                   : false
    },
    
    
    after : {
        
        initialize : function () {
            var map     = this.map  = this.map != null ? this.map : (this.name == 'INDEX' ? '' : this.name) 
            
            this.tokens = this.stringToTokens(map)
        }
    },
    
    
    methods : {
        
        prepareApply : function (target) {
        },
        
        
        apply : function (target) {
        },
        
        
        unapply : function (from) {
        },
        
        
        stringToTokens : function (path) {
            var me          = this
            var tokens      = []
            var segments    = path.replace(/\/+/, '/').split('/')
            
            Joose.A.each(segments, function (token, index) {
                
                if (token == '.' && !index) return

                tokens.push(me.createToken(token, index, segments.length))
            })
            
            return tokens
        },
        
        
        isIndex : function () {
            var lastToken = this.tokens[ this.tokens.length - 1 ]
            
            return lastToken instanceof Symbie.Meta.Route.Token.String && lastToken.token == ''
        },
        
        
        // route can be local, so this method should return tokens for passed prefix, which forms a absolute path
        // prefix will also have trailing `/` symbol, so remove it to avoid extra token
        getTokensFor : function (prefix) {
            if (this.tokens[0] instanceof Symbie.Meta.Route.Token.Root) return [].concat(this.tokens)
            
            return this.stringToTokens(prefix.replace(/\/$/, '')).concat(this.tokens)
        },
        
        
        createToken : function (token, index, length) {
            if (token == '' && !index && length > 1)    return new Symbie.Meta.Route.Token.Root()
            
            if (/^\*/.test(token))                      return new Symbie.Meta.Route.Token.WildCard()
            
            if (/^:/.test(token)) {
                var name = /^:(.*)/.exec(token)[1]
                
                var regex = this.where && this.where[name] || /(.*)/
                
                return new Symbie.Meta.Route.Token.Parameter({ token : token, name : name, regex : regex })
            }
            
            return new Symbie.Meta.Route.Token.String({ token : token })
        },
        
        
        asString : function (parameters) {
            var res = []
            
            Joose.A.each(this.tokens, function (token, index) {
                var str = token.asString(parameters)
                
                if (str != null) res.push(str)
            })
            
            return res.join('/')
        },
        
        
        doExecute : function (context) {
            var args = [ context ]
            
            Joose.A.each(context.getParamsOrder(), function (paramName) {
                
                args.push(context.getParam(paramName))
            })
            
            return this.execute.apply(this, args)
        }
    },
    
    
    continued : {
        
        methods : {
            
            prepare : function () {
                var CONTINUE = this.getCONTINUE()
                
                var me      = this
                var deps    = this.use
                
                if (deps)
                    use(deps, function () {
                        
                        me.ready = true
                        
                        CONTINUE()
                    })
                else {
                    me.ready = true
                    
                    CONTINUE()
                }
            },
            
            
            execute : function (context) {
                if (!this.ready) this.prepare()   // async
                
                this.TRY(this.action, context.controller, arguments).now()
            }
        }
    }
})
;
Class('Symbie.Meta.Route.Match', {
    
    has : {
        tokens                  : { required : true },
        route                   : { required : true },
        controller              : { required : true },
        
        currentTokenIndex       : 0,
        
        parameters              : Joose.I.Object,
        paramsOrder             : Joose.I.Array
    },
    
    
    methods : {
        
        asString : function (params) {
            var str = this.route.asString(params)
            
            if (/^\//.test(str)) return str
            
            return this.controller.getFullPrefix() + str
        },
        
        
        consumeToken : function (token, index, length) {
            var currentToken    = this.tokens[ this.currentTokenIndex ]
            var parameters      = this.parameters
            var paramsOrder     = this.paramsOrder
            
            
            if (currentToken instanceof Symbie.Meta.Route.Token.WildCard) {
                if (!parameters.splat) {
                    parameters.splat    = []
                    
                    paramsOrder.push('splat')
                }
                
                parameters.splat.push(token)
                
                return true
            }
            
            this.currentTokenIndex++
            
            if (currentToken instanceof Symbie.Meta.Route.Token.String)
                return currentToken.match(token)
            
            if (currentToken instanceof Symbie.Meta.Route.Token.Parameter) {
                var match = currentToken.match(token)
                
                if (match) {
                    var paramName = currentToken.name
                    
                    
                    paramsOrder.push(paramName)
                    
                    if (match[1] != null && match[2] != null) 
                        parameters[paramName] = match
                    else
                        if (match[1] != null)
                            parameters[paramName] = match[1]
                        else
                            parameters[paramName] = token
                }
                
                return match
            }
            
            return false
        },
        
        
        compareBySpecificity : function (another, pathLength) {
            
            var thisTokens      = this.tokens
            var anotherTokens   = another.tokens
            
            var thisLength      = thisTokens.length
            var anotherLength   = anotherTokens.length
            
            // weight index routes with extra token from actual path a bit less 
            if (thisLength == pathLength && anotherLength == pathLength + 1 && another.route.isIndex())
                return 5
            
            if (anotherLength == pathLength && thisLength == pathLength + 1 && this.route.isIndex())
                return -5
            
            if (thisLength < anotherLength) return -1
            if (thisLength > anotherLength) return 1
            
            for (var i = 0; i < thisTokens.length; i++ ) {
                
                var specDiff = thisTokens[i].specificity - anotherTokens[i].specificity
                
                if (specDiff) return specDiff
            }
            
            return 0
        }
    }
    
})
;
Class('Symbie.Context', {
    
    trait : 'JooseX.CPS',
    
    has : {
        dispatchedFrom          : { required : true },
        dispatchParams          : { required : true },
        
        controller              : { required : true },
        match                   : { required : true },
        
        path                    : null,
        query                   : null,
        
        compChain               : Joose.I.Array,
        
        stash                   : Joose.I.Object
    },
    
    
    methods : {
        
        get : function (name) {
            return this.stash[ name ]
        },
        
        
        set : function (name, value) {
            this.stash[ name ] = value
        },
        
        
        app : function () {
            return this.getRoot()
        },
        
        
        getRoot : function () {
            return this.compChain[0]
        },
        
        
        initialize : function () {
            this.controller.getParents(this.compChain)
        },
        
        
        getParameters : function () {
            return this.match.parameters
        },
        
        
        getParams : function () {
            return this.getParameters()
        },
        
        
        getParamsOrder : function () {
            return this.match.paramsOrder
        },
        
        
        getParam : function (name) {
            return this.getParameters()[ name ]
        },
        
        
        getSplat : function () {
            return this.match.path
        },
        
        
        getRoute : function () {
            return this.match.route
        },
        
        
        eachComp : function (func, scope) {
            return Joose.A.each(this.compChain, func, scope || this)
        },
        
        
        eachCompR : function (func, scope) {
            var compChain = this.compChain
            
            for (var i = compChain.length - 1; i >= 0; i--)
                if (func.call(scope || this, compChain[i], i) === false) return false
        },
        
        
        runACTIVATE : function () {
            
            this.eachComp(function (controller) {
                
                controller.ACTIVATE(this)
            })
        },
        
        
        runFINALIZE : function () {
            
            this.eachCompR(function (controller) {
                
                controller.FINALIZE(this)
            })
        }
    },
    
    
    continued : {
        
        methods : {
            
            call : function (params) {
                var dispatchedFrom      = this.dispatchedFrom
                
                var match               = dispatchedFrom.findMatch(params)
                
                match.route.doExecute(this).now()
            },
            
            
            visit : function (path) {
//                this.redirected = true
//                
//                var me = this
//                
//                this.router.dispatch({
//                    routePath       : path,
//                    prevContext     : this
//                }).andThen(function () {
//                    
//                    this.CONTINUE(me)
//                })
            },
            
            
            redirect : function (path) {
//                this.redirected = true
//                
//                var me = this
//                
//                this.router.dispatch({
//                    routePath       : path,
//                    prevContext     : this
//                }).andThen(function () {
//                    
//                    this.CONTINUE(me)
//                })
            },
            
            
            run : function () {
                var me          = this
                var route       = this.getRoute()
                var controller  = this.controller
                
                this.runPRE().andThen(function (stop) {
                    
                    if (stop) {
                        this.CONTINUE(me)
                        
                        return
                    }
                    
                    this.runACTIVATE()      // sync
                    
                    controller.BEGIN(this)  // async
                    
                    route.doExecute(this)     // async
                    
                    controller.END(this).andThen(function () {
                        
                        me.runFINALIZE()    // sync
                        
                        this.CONTINUE(me)
                    })
                })
            },
            
            
            runPRE : function () {
                
                this.eachComp(function (controller) {
                    
                    controller.PRE(this).then(function (stop) {
                        
                        if (stop === false) 
                            this.RETURN(true)
                        else
                            this.CONTINUE()
                    })
                })
                
                this.NOW()
            }
        }
    }
    
});
Role('Symbie.Meta.Controller', {
    
    use : [ 'Symbie.Meta.Route' ],
    
    
    has : {
        prefix          : '',
        
        controllers     : Joose.I.Object,
        config          : Joose.I.Object,
        
        routes          : null,
        routesMC        : Joose.I.FutureClass('Symbie.Meta.Route')
    },
    
    
    methods : {
        
        hasRoute : function (name) {
            return this.stem.properties.routes.haveProperty(name)
        },
        
        
        hasOwnRoute : function (name) {
            return this.stem.properties.routes.haveOwnProperty(name)
        },
        
        
        getRoute : function (name) {
            return this.stem.properties.routes.getProperty(name)
        },
        
        
        getRoutes : function () {
            return this.stem.properties.routes
        }
    },
    
    
    after : {
        
        initInstance : function (instance, config) {
            var controllers = instance.controllers
            
            Joose.O.each(this.controllers, function (params, name) {
                
                controllers[ name ] = new (Joose.S.strToClass(name))(Joose.O.copy(params, { parent : instance }))
            })
        }
    },
    
    
    before : {
        
        // move the information from `plugins` and `controllers` to meta 
        prepareProps : function (props) {
            var does    = props.does    = [].concat(props.does || [])
            
            var config  = this.config   = props.config || {}
            delete props.config
            
            Joose.A.each(Joose.O.wantArray(props.plugins || []), function (pluginsDesc) {
                
                if (typeof pluginsDesc == 'string')
                    does.push(Joose.S.strToClass(pluginsDesc))
                else
                    Joose.O.each(pluginsDesc, function (params, name) {
                        does.push(Joose.S.strToClass(name))
                        
                        config[ name ] = params
                    })
            })
            
            delete props.plugins
            
            var controllers = this.controllers
            
            Joose.A.each(Joose.O.wantArray(props.controllers || []), function (value) {
                
                if (typeof value == 'string')
                    controllers[ value ] = {}
                else
                    Joose.O.each(value, function (params, name) {
                        controllers[ name ] = params
                    })
            })
            
            delete props.controllers
        },
        
        
        processStem : function () {
            var superMeta = this.superClass.meta
            
            this.routes = Joose.O.getMutableCopy(superMeta.routes || {})
        }
    },
    
    
    stem : {
        
        has     : {
            controllerInitialized       : false
        },
        
        // modifier can be applied several times
        after : {
            
            initialize : function () {
                if (!this.controllerInitialized) {
                    
                    this.controllerInitialized  = true
                    
                    // not just push, because this property will be actually taken from the prototype, so we shouldn't modify it
                    this.processOrder           = this.processOrder.concat('routes')
                    
                    var targetMeta              = this.targetMeta
                    
                    //will be Joose.Managed.PropetySet.Mutable
                    this.addProperty('routes', {
                        properties : targetMeta ? targetMeta.routes : {}
                    })
                }
            }
        }
    },
    
    
    builder : {
        
        methods : {
            
            routes : function (meta, info) {
                var routes = meta.stem.properties.routes
                
                Joose.O.each(info, function (props, name) {
                    if (typeof props == 'function') props = {
                        action : props
                    }
                    
                    props.meta = props.meta || meta.routesMC
                    
                    routes.addProperty(name, props)
                })
            },
            
            
            prefix  : function (meta, info) {
                meta.prefix = info
            }
        }
    }
})



JooseX.Namespace.Depended.meta.extend({
    
    methods : {
        
        symbieCollect : function (data, deps) {
            Joose.A.each(Joose.O.wantArray(data), function (desc) {
                
                if (typeof desc == 'string')
                    deps.push(desc)
                else
                    Joose.O.each(desc, function (params, name) {
                        deps.push(name)
                    })
            })
        }
        
    },
    
    
    after : {
        
        alsoDependsFrom : function (extend, summaredDeps) {
            
            if (extend.plugins)         this.symbieCollect(extend.plugins, summaredDeps)
            if (extend.controllers)     this.symbieCollect(extend.controllers, summaredDeps)
        }
    }
})
;
Class('Symbie.Controller', {
    
    traits      : [ 
        'JooseX.CPS',
        
        'Symbie.Meta.Controller'
    ],
    

    use         : [ 
        'Symbie.Context', 
        'Symbie.Meta.Route.Match' 
    ],
    
    
    does    : [
        'JooseX.Observable'
    ],
    
    
    has : {
        prefix                  : { is : 'rw' },
        fullPrefix              : { is : 'rw' },
        
        parent                  : { required : true },
        
        // will be filled by meta
        controllers             : Joose.I.Object,
        
        contextClass            : Joose.I.FutureClass('Symbie.Context')
    },
    
    
    methods : {
        
        getConfigFor : function (name) {
            return this.meta.config[ name ]
        },
        
        
        getParents : function (res) {
            res.unshift(this)
            
            if (this.parent) this.parent.getParents(res)
        },
        
        
        getRoot : function () {
            return this.parent && this.parent.getRoot() || this
        },
        
        
        // default value for prefix is being taken from meta
        // leading `./` is being stripped
        // missing trailing `/` is being added
        getPrefix : function () {
            var prefix      = this.prefix || this.meta.prefix
            
            return prefix.replace(/^\.\//, '').replace(/\/?$/, '/')
        },
        
        
        getFullPrefix : function () {
            var ownPrefix   = this.getPrefix()
            
            if (/^\//.test(ownPrefix)) return ownPrefix
            
            var prefix      = this.parent && this.parent.getFullPrefix() || '/'
            
            return prefix + ownPrefix
        },
        
        
        getOwnRoute : function (name) {
            return this.meta.getRoute(name)
        },
        
        
        getOwnRoutes : function () {
            return this.meta.getRoutes()
        },
        
        
        forEachController : function (func, scope) {
            return Joose.O.each(this.controllers, func, scope || this)
        },
        
        
        forEachRoute : function (func, scope) {
            var me      = this
            var self    = scope || this
            
            if (this.getOwnRoutes().each(function (route, name) {
                
                return func.call(self, route, name, me)
                
            }) === false) return false
            
            
            return this.forEachController(function (controller) {
                
                return controller.forEachRoute(func, scope)
            })
        },
        
        
        
        getMatchesPopulation : function (maxLength) {
            var matches = []
            
            this.forEachRoute(function (route, name, controller) {
                
                var tokens  = route.getTokensFor(controller.getFullPrefix())
                
                // add routes which has not much than `maxLength` tokens to population
                // exception is the index routes, which are allowed to have an extra token at the end
                if (tokens.length <= maxLength || tokens.length == maxLength + 1 && route.isIndex()) matches.push(new Symbie.Meta.Route.Match({
                    
                    controller      : controller,
                    tokens          : tokens,
                    route           : route    
                }))
                
            }, this)
            
            return matches
        },
        
        
        filterMatches : function (token, matches, index, length) {
            
            var filtered = []
            
            Joose.A.each(matches, function (match) {
                
                var survived = match.consumeToken(token, index, length)
                
                if (survived) filtered.push(match)
            })
            
            return filtered
        },
        
        
        findMatches : function (path) {
            var tokens = path.split('/')
            
            var matches = this.getMatchesPopulation(tokens.length)
            var length  = tokens.length
            
            Joose.A.each(tokens, function (token, index) {
                
                matches = this.filterMatches(token, matches, index, length)
            }, this)
            

            return matches
        },
        
        
        findMatch : function (path) {
            var matches     = this.findMatches(path)
            var length      = path.split('/').length
            
            if (!matches.length) throw "Can't find route for the path [" + path + "]"
            
            matches.sort(function (matchA, matchB) {
                return matchB.compareBySpecificity(matchA, length)
            })
                
            if (matches.length > 1 && matches[0].compareBySpecificity(matches[1], length) == 0) throw "Ambiguous route match"
                
            return matches[0]
        },
        
        
        parseQuery : function (queryStr) {
            var obj     = {}
            
            Joose.A.each(queryStr.split('&'), function (pair) {
                if (!pair) return
                
                var res = /([^=]*)(?:=(.*))?/.exec(pair)
                
                var name    = decodeURIComponent(res[1])
                var value   = decodeURIComponent(res[2]) || ''
                
                var already = obj[ name ]
                
                obj[ name ] = already != null ? [].concat(already, value) : value
            })
            
            return obj
        },
        
        
        ACTIVATE : function () {
        },
        
        
        FINALIZE : function () {
        }
    },
    
    
    continued : {
    
        methods : {

            PRE     : function () {
                this.CONTINUE()
            },
            
            
            BEGIN   : function () {
                this.CONTINUE()
            },
            
            
            END     : function () {
                this.CONTINUE()
            },
            
            
            dispatch : function (params) {
                if (typeof params == 'string') params = { path : params }
                
                //extracting trailing parameteres after '?'
                var res = /([^\?]*)(?:\?(.*))?/.exec(params.path)
                
                var path    = res[1]
                var query   = this.parseQuery(res[2] || '')
                
                try {
                    var match = this.findMatch(path)
                } catch (e) {
                    
                    if (this.fireEvent('dispatchException', e) !== false) 
                        throw e
                    else {
                        this.CONTINUE()
                        
                        return
                    }
                }
                
                var context = new this.contextClass({
                    dispatchParams      : params,
                    dispatchedFrom      : this,
                    
                    controller          : match.controller,
                    match               : match,
                    
                    query               : query,
                    path                : path
                })
                
                context.run().CATCH(function (e) {
                    
                    if (this.fireEvent('dispatchException', e) !== false) 
                        throw e
                    else
                        this.CONTINUE(context)
                    
                }, this).now()
            },
            
            
            setup : function () {
                
                this.forEachController(function (controller) {
                    
                    this.AND(function () {
                        controller.setup().now()
                    })
                })
                
                this.andTHEN(function () {
                    
                    this.fireEvent('ready', this)
                    
                    this.CONTINUE()
                })
            }
        }
    }
})
;
Class('Symbie.Application', {
    
    isa     : 'Symbie.Controller',
    

    has : {
        parent                  : null,
        prefix                  : '/'
    },
    
    
    methods : {
        
    },
    
    
    continued : {
        
        methods : {
            
            launch : function (path) {
                this.dispatch(path).now()
            },
            
            
            run : function (path) {
                this.setup()        // async
                
                this.launch(path || '/').now()
            }
        }
    }
    
});
Class('Symbie', {
    
    use     : [ 'Symbie.Application' ]
    
});
